/*
 * (c) Copyright 2015 -- Anders Torger
 *
 * This program is open source. For license terms, see the LICENSE file.
 *
 */
#ifndef GAMUT_H_
#define GAMUT_H_

#include <matmath.h>

struct gamut_spec {
    bool locus_enabled;
    bool xyz2rgb_enabled;
    m3x3 xyz2rgb;
    double chroma_scale;
};

struct gamut_compression {
    double hi_rgb_lim;
    double lo_rgb_lim;
    struct gamut_spec inner;
    struct gamut_spec dest;
    struct gamut_spec clip;
};

v3
gamut_compress(const struct gamut_compression *gc,
               const m3x3 rgb2xyz,
               const m3x3 xyz2rgb,
               v3 xyz,
               v3 wp,
               void *lcms2_h02);

m3x3
gamut_remap_xyz2rgb(m3x3 xyz2rgb,
                    v3 wp);

bool
gamut_isvalid(v3 xyz);

double
gamut_max_valid_chroma(v3 jch,
                       v3 wp,
                       double clip_hi_limit,
                       void *lcms2_h02);

v3
gamut_clip_to_valid(v3 xyz,
                    v3 wp);

v3
gamut_acr_rgb_clip(v3 rgb);

v3
gamut_saturated_rgb_clip(v3 rgb);

void
gamut_init(const struct observer *obs,
           const spectrum_t *illuminant);

#endif
